/*
 * Text Formatting Extension
 * 
 * by Alexander Schuc
 * http://blog.furred.net
 * 
 * for BlogEngine.net (http://www.dotnetblogengine.net)
 * 
 * using Markdown.NET
 *	http://www.aspnetresources.com/blog/markdown_announced.aspx
 * 
 * Infos on Markdown
 *	http://daringfireball.net/projects/markdown/
 * 
 * Text Formatting Extension is licensed under Ms-PL
 * http://www.microsoft.com/resources/sharedsource/licensingbasics/permissivelicense.mspx
 * 
 * For Markdown license please take look into 'markdown_license.txt'
 * which comes with this file.
 * 
 * Version History
 *	v0.2     - 04.08.2007 - Added support for trackbacks,
 *	                        using ExtensionManager for settings
 *	v0.1     - 03.12.2007 - Initial Version
 * 
 * Installation:
 *	-	put 'TextFormatter.cs' into your extension directory
 *	-	Every post which should be formatted has to start with '<md>' (without ')!
 *		This behaviour can be disabled by setting 'formatOnlyMarked' to false
 *	-	To format comments set 'formatComments' to true
 * 
 * Best use without tinyMCE or some other WYSIWYG editor!
 */

using System;
using anrControls;
using BlogEngine.Core;
using BlogEngine.Core.Web.Controls;
using BlogEngine.Core.Ping;
using System.Threading;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Web;
using System.Net;

/// <summary>
/// Provides Markdown formatting for Pages and Posts.
/// </summary>
[Extension("Markdown formatting for BlogEngine.net", "0.2", "Alexander Schuc <aschuc@furred.net>")]
public class TextFormatter
{
	private bool formatOnlyMarked = true;
	private bool formatComments = false;
	private bool sendPings = true;
	private string marker = "<md>";

	private ExtensionSettings settings = null;

	private readonly Markdown markdown;
	private readonly SmartyPants smartypants;
	private static readonly Regex markdownLinksRegex = new Regex(@"\[.*?\](?:\((?<uri>.*?)(?:"".*?"")?\)|: (?<uri>.*?)(?:"".*?"")?$)", RegexOptions.IgnoreCase | RegexOptions.Compiled);
	private static readonly Regex trackbackLinkRegex = new Regex("trackback:ping=\"([^\"]+)\"", RegexOptions.IgnoreCase | RegexOptions.Compiled);

	private static string StorageLocation
	{
		get
		{
			string p = BlogSettings.Instance.StorageLocation.Replace("~/", "");
			return System.IO.Path.Combine(System.Web.HttpRuntime.AppDomainAppPath, p);
		}
	}

	public TextFormatter()
	{
		markdown = new Markdown();
		smartypants = new SmartyPants();

		SetupSettings();

		Page.Serving += new EventHandler<ServingEventArgs>(Serving);
		Post.Serving += new EventHandler<ServingEventArgs>(Serving);

		Page.Saved += new EventHandler<SavedEventArgs>(Saved);
		Post.Saved += new EventHandler<SavedEventArgs>(Saved);

		if (formatComments)
			Comment.Serving += Serving;
	}

	private void SetupSettings()
	{
		ExtensionSettings settings_setup = new ExtensionSettings("TextFormatter");
		settings_setup.IsScalar = true;
		settings_setup.AddParameter("formatOnlyMarked", "Format only marked entries");
		settings_setup.AddParameter("marker", "Marker for entries");
		settings_setup.AddParameter("formatComments", "Format comments");
		settings_setup.AddParameter("sendPings", "Send Track-/PingBacks");

		settings_setup.AddValue("formatOnlyMarked", true);
		settings_setup.AddValue("marker", "<md>");
		settings_setup.AddValue("formatComments", false);
		settings_setup.AddValue("sendPings", true);

		ExtensionManager.ImportSettings("TextFormatter", settings_setup);
		
		settings = ExtensionManager.GetSettings("TextFormatter");

		LoadData();
	}

	private void LoadData()
	{
		formatOnlyMarked = bool.Parse(settings.GetSingleValue("formatOnlyMarked"));
		marker = settings.GetSingleValue("marker");
		formatComments = bool.Parse(settings.GetSingleValue("formatComments"));
		sendPings = bool.Parse(settings.GetSingleValue("sendPings"));
	}

	private void Serving(object sender, ServingEventArgs e)
	{
		if (HttpContext.Current != null
			&& HttpContext.Current.Items["textFormatterLoaded"] == null)
		{
			HttpContext.Current.Items["textFormatterLoaded"] = true;
			LoadData();
		}

		if (formatOnlyMarked && !e.Body.StartsWith(marker))
		{
			return;
		}

		string text = formatOnlyMarked ? e.Body.Substring(marker.Length) : e.Body;

		text = markdown.Transform(text);
		text = smartypants.Transform(text, ConversionMode.EducateDefault);

		e.Body = text;
	}

	private void Saved(object sender, SavedEventArgs e)
	{
		if (!sendPings)
			return;

		if (!BlogSettings.Instance.EnableTrackBackSend
			&& !BlogSettings.Instance.EnablePingBackSend)
			return;

		IPublishable item = (IPublishable)sender;

		if ((HttpContext.Current == null || !HttpContext.Current.Request.IsLocal) && item.IsVisible)
		{
			Uri url = item.AbsoluteLink;
			ThreadPool.QueueUserWorkItem(delegate { Ping(item, url); });
		}
	}

	private void Ping(IPublishable item, Uri itemUrl)
	{
		System.Threading.Thread.Sleep(1000);

		foreach (Uri url in GetUrlsFromContent(item.Content))
		{
			string pageContent = ReadFromWeb(url);
			Uri trackbackUrl = GetTrackBackUrlFromPage(pageContent);
			bool isTrackbackSent = false;

			if (trackbackUrl != null)
			{
				TrackbackMessage message = new TrackbackMessage(item, trackbackUrl, itemUrl);
				isTrackbackSent = Trackback.Send(message);
			}

			if (!isTrackbackSent)
			{
				Pingback.Send(itemUrl, url);
			}
		}
		
	}

	private static Uri GetTrackBackUrlFromPage(string input)
	{
		string url = trackbackLinkRegex.Match(input).Groups[1].ToString().Trim();
		Uri uri;

		if (Uri.TryCreate(url, UriKind.Absolute, out uri))
			return uri;
		else
			return null;
	}

	private static List<Uri> GetUrlsFromContent(string content)
	{
		List<Uri> urlsList = new List<Uri>();
		foreach (Match myMatch in markdownLinksRegex.Matches(content))
		{
			string url = myMatch.Groups["uri"].ToString().Trim();
			Uri uri;
			if (Uri.TryCreate(url, UriKind.Absolute, out uri))
				urlsList.Add(uri);
		}

		return urlsList;
	}

	private static string ReadFromWeb(Uri sourceUrl)
	{
		string html;
		using (WebClient client = new WebClient())
		{
			client.Headers.Add("User-Agent", "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0)");
			html = client.DownloadString(sourceUrl);
		}
		return html;
	}
}
